SCHEDULE_PLACEHOLDER = "Generate a schedule to see the weekly view."
DETAIL_PLACEHOLDER = "Enter a course code to see details here."

ACTION_DETAIL = "action-detail"
ACTION_SCHEDULE = "action-schedule"
ACTION_QUIT = "action-quit"

SIDEBAR_TITLE_DETAIL = "Course Detail"
SIDEBAR_TITLE_SCHEDULE = "Weekly Schedule"
